import assert from 'assert';
import Q from 'q';

/**
 * @param milliseconds duration to wait
 * @returns a promise that fulfills no less than `milliseconds` after timeout() was called
 */
async function timeout(milliseconds:number):Promise<void> {}
// implemented three ways below:

// version 1:
// using a Q.Deferred object, 
//  which consists of a promise and its resolve and reject operations:
async function timeout1(milliseconds:number):Promise<void> {
    const deferred: Q.Deferred<void> = Q.defer();
    setTimeout(function() {
        deferred.resolve(); // this mutates deferred.promise into the fulfilled state
        // or we could call deferred.reject(new Error(...)) to reject the promise instead
    }, milliseconds);
    return deferred.promise;
}

// version 2:
// implemented using Promise constructor,
// explicitly pulling out its resolve and reject operations:
async function timeout2(milliseconds:number):Promise<void> {
    let resolvePromise;
    let rejectPromise;
    const promise = new Promise<void>(function(resolve, reject) {
        resolvePromise = resolve;
        rejectPromise = reject;
    });
    assert(resolvePromise);
    setTimeout(resolvePromise, milliseconds);
    return promise;
}

// version 3:
// implemented using Promise constructor, more concisely
async function timeout3(milliseconds:number):Promise<void> {
    return new Promise<void>(function(resolve, reject) {
        setTimeout(resolve, milliseconds);
    })
}
